;Constant list
.include "HDL50001_pcf.inc"
.alias nextPA 2
.alias valuePA 3
.alias TRbyte 4
.alias WRbyte 5
.alias Adr 6
.alias DM_START		0x00000000 ;start DM address for output by uart0
.alias BLOCK_SIZE	0x00000800 ;end DM address for output
.alias iterator1 0
.alias iterator2 1
.alias byte_DM 7

.text

start:
	jmp initUART0
	getl 0x00000300
	wrl @1, GPIOB_BPS ;allow port alternative function uart0
	getl 0x00C00000
	wrl @1, GPIOA_BPS ;allow port alternative function i2c0
	getl 0x00000104
	wrl @1, UART0_BDR ;uart frequency
	getl 0x00000080 
	wrl @1, I2C0_CR ;enable i2c
	getl 0x000000A0
	wrl @1, I2C0_CPS ;clock i2c 100kHz
	setl #valuePA, i2c_adr1 ;value for next paragraph
	getb 0xA0
	setb #Adr, @1 ;set slave address for write 
complete

initUART0:
	jmp i2c_slv1
	getl 0x0000000A
	wrl @1, UART0_DATA
	getl 0x00000003
	wrl @1, UART0_CR
complete

;read byte from i2c memory address 0x0000
i2c_slv1:
	getb #Adr
	wrb @1, I2C0_TDATA ;send Slave Address
	setl #nextPA, #valuePA ;set next paragraph
	jmp i2c_slv2
complete

i2c_slv2:
	getl 0x00000090
	wrl @1, I2C0_CMD ;send START and write slave address
	jmp end_trb
complete

;wait end of data transmit
end_trb:
	getl 0x00000002
	rdl I2C0_ST
	and @1, @2
	je @1, ACK
	jne @2, end_trb
complete

;wait end of data transmit after STOP
end_trb_stop:
	getl 0x00000002
	rdl I2C0_ST
	and @1, @2
	je @1, #nextPA
	jne @2, end_trb_stop
complete

;wait for ack 
ACK:
	getl 0x00000080
	rdl I2C0_ST
	and @1, @2
	je @1, #nextPA
	jne @2, error_ACK
complete

i2c_adr1:
	getb 0x00
	wrb @1, I2C0_TDATA ;send first adress byte
	setl #nextPA, i2c_adr2
	jmp i2c_wb
complete

i2c_wb:
	getl 0x00000010
	wrl @1, I2C0_CMD ;write byte
	jmp end_trb
complete

i2c_adr2:
	getb 0x00
	wrb @1, I2C0_TDATA ;send second adress byte
	getb 0xA1
	setb #Adr, @1 ;set slave address for read
	setl #nextPA, i2c_slv1
	setl #valuePA, NACK
	jmp i2c_wb
complete

;send NACK (last byte)
NACK:
	getl 0x00000068
	wrl @1, I2C0_CMD ;send NACK to slave device
	setl #nextPA, i2c_dbyte
	jmp end_trb_stop 
complete

i2c_dbyte:
	rdl I2C0_RDATA ; read byte
	setl #TRbyte, @1
	wrl @2, 0x00000001 ; TEST write data to DM
	jmp buf_TXD
complete


;;;;; Uart sending ;;;;;

;RXD handler
Rxd_int:
    jmp i2c_slv1 ;up to write to i2c memory
	rdl UART0_DATA
	setl #WRbyte, @1
	getb 0xA0
	setb #Adr, @1
	setl #valuePA, i2c_wradr1
complete

;test FIFO on full
buf_TXD:
	rdl UART0_ST
	getl 0x00000200
	and @1, @2
	jne @1, buf_TXD
	je @2, Send_byte
complete

;Send byte from i2c memory
Send_byte:
	jmp MainLoop
	getb #TRbyte
	wrb @1, UART0_DATA
complete

;;Loop (wait for receive bits)
MainLoop:
	rdl UART0_ST
	getl 0x00000001
	and @2,@1
	je @1, MainLoop
	jne @2, Rxd_int 
complete

i2c_wradr1:
	getb 0x00
	wrb @1, I2C0_TDATA ;send first adress byte
	setl #nextPA, i2c_wradr2
	jmp i2c_wb
complete

i2c_wradr2:
	getb 0x00
	wrb @1, I2C0_TDATA ;send first adress byte
	setl #nextPA, i2c_wrdbyte
	jmp i2c_wb
complete

i2c_wrdbyte:
	getl #WRbyte
	wrl @1, I2C0_TDATA ; write byte
	setl #nextPA, i2c_wrstop
	jmp i2c_wb
complete

i2c_wrstop:
	jmp stop
	getl 0x00000050
	wrl @1, I2C0_CMD ;write byte and STOP
complete

stop:
	jmp DM_download
	getl 0x20000000
	wrl @1, GPIOB_DIR
	getl 0x20000000
	wrl @1, GPIOB_OUT 
complete

error_ACK:
	getl #nextPA
	wrl @1, UART0_DATA
	getl 0x60000000
	wrl @1, GPIOB_DIR
	getl 0x60000000
	wrl @1, GPIOB_OUT 
complete

DM_download:
	jmp read_DM
	setl #iterator1, 0x00000000
complete

read_DM:
	getl BLOCK_SIZE
	subl @1, #iterator1
	je @1, stop2
	jne @2, out_DM
	getl #iterator1
	addl @1, DM_START
	rdq @1
	setq #byte_DM, @1
	addl @4, 0x00000008
	setl #iterator1, @1
	setl #iterator2, 0
complete

out_DM:
	getl #iterator2
	getl 56
	subl @1, @2
	je @1, buf_TXD2
	jne @2, out_DM
	getq #byte_DM
	slrq @1, #iterator2
	wrq @1, UART0_DATA
	getl 8
	addl @1, #iterator2
	setl #iterator2, @1
complete

buf_TXD2:
	rdl UART0_ST
	getl 0x00000004
	and @1, @2
	je @1, buf_TXD2
	jne @2, read_DM
complete

;enable led 2,3
stop2:
	getl 0x60000000
	wrl @1, GPIOB_DIR
	getl 0x60000000
	wrl @1, GPIOB_OUT
complete


















