//USB constant file
#ifndef USB_H
#define USB_H

#include "HDL51001_ccf.h"
#include "HDL50001_gpio.h"

typedef struct
{
  __IO uint32_t HSCR;
  __IO uint32_t RESERVED0;
  __IO uint32_t EPCR0;
  __IO uint32_t EPST0;
  __IO uint32_t EPTRST0;
  __IO uint32_t EPNTRST0;
  __IO uint32_t EPCR1;
  __IO uint32_t EPST1;
  __IO uint32_t EPTRST1;
  __IO uint32_t EPNTRST1;
  __IO uint32_t EPCR2;
  __IO uint32_t EPST2;
  __IO uint32_t EPTRST2;
  __IO uint32_t EPNTRST2;
  __IO uint32_t EPCR3;
  __IO uint32_t EPST3;
  __IO uint32_t EPTRST3;
  __IO uint32_t EPNTRST3;
  __IO uint32_t CR;
  __IO uint32_t LNST;
  __IO uint32_t INT;
  __IO uint32_t MSKINT;
  __IO uint32_t ADDR;
  __IO uint32_t MSPFRAME;
  __IO uint32_t LSPFRAME;
  __IO uint32_t RXDATA0;
  __IO uint32_t RXMSB0;
  __IO uint32_t RXLSB0;
  __IO uint32_t RXCLR0;
  __IO uint32_t TXDATA0;
  __IO uint32_t TXCLR0;
  __IO uint32_t RXDATA1;
  __IO uint32_t RXMSB1;
  __IO uint32_t RXLSB1;
  __IO uint32_t RXCLR1;
  __IO uint32_t TXDATA1;
  __IO uint32_t TXCLR1;
  __IO uint32_t RXDATA2;
  __IO uint32_t RXMSB2;
  __IO uint32_t RXLSB2;
  __IO uint32_t RXCLR2;
  __IO uint32_t TXDATA2;
  __IO uint32_t TXCLR2;
  __IO uint32_t RXDATA3;
  __IO uint32_t RXMSB3;
  __IO uint32_t RXLSB3;
  __IO uint32_t RXCLR3;
  __IO uint32_t TXDATA3;
  __IO uint32_t TXCLR3;
  
} USB_TypeDef;

#define AHBPERIPH_BASE       (0xFFF00000)
//USB
#define USB_BASE            (AHBPERIPH_BASE + 0x00014000)
#define USB		         	((USB_TypeDef *) USB_BASE)


//usb constant and macro
//  (10 ) 
#define USB_HSCR() USB->HSCR = 2
//     EP
#define USB_EPCR(EN, READY, OUT, STALL, ISO, EPCRN) USB->EPCRN = (EN & 0x01)|((READY & 0x01) << 1)|((OUT & 0x01) << 2)|((STALL & 0x01) << 3)|((ISO & 0x01) << 4)
//  STALL
#define USB_STALL(EPCRN) USB->EPCRN |= 8
// EP   (0 - EP  , 1 -  EP )
#define USB_READY(EPCRN) USB->EPCRN |= 2 
//   CRC (0 -  ; 1 -  )
#define USB_EPST_CRC(EPSTN) ((USB->EPSTN & 1)? 1 : 0)
//     (0 -  ; 1 -  )
#define USB_EPST_STRUCT(EPSTN) ((USB->EPSTN & 2)? 1 : 0)
//       () (0 -   ; 1 -  )
#define USB_EPST_OVF(EPSTN) ((USB->EPSTN & 4)? 1 : 0)
//    Host (0 -  ; 1 -    Host)
#define USB_EPST_TIME(EPSTN) ((USB->EPSTN & 8)? 1 : 0)
//NACK   Host(0 -  ; 1 - )
#define USB_EPST_NACK(EPSTN) ((USB->EPSTN & 16)? 1 : 0)
//STALL   Host(0 -  ; 1 - )
#define USB_EPST_STALL(EPSTN) ((USB->EPSTN & 32)? 1 : 0)
//ACK   Host(0 -  ; 1 - )
#define USB_EPST_ACK(EPSTN) ((USB->EPSTN & 64)? 1 : 0)
//  (0 - DATA0; 1 - DATA1)
#define USB_EPST_DATA(EPSTN) ((USB->EPSTN & 128)? 1 : 0)
//    SETUP(0 - ; 1 - )
#define USB_EPTRST_SETUP(EPTRSTN) (((USB->EPTRSTN) == 0)? 1 : 0)
//    IN(0 - ; 1 - )
#define USB_EPTRST_IN(EPTRSTN) (((USB->EPTRSTN == 1)? 1 : 0)
//    OUT_DATA(0 - ; 1 - )
#define USB_EPTRST_OUT(EPTRSTN) (((USB->EPTRSTN == 2)? 1 : 0)
//  ,   NAK  Host,  SETUP(0 - ; 1 - )
#define USB_EPNTRST_SETUP(EPNTRSTN) (((USB->EPNTRSTN) == 0)? 1 : 0)
//  ,   NAK  Host,  IN(0 - ; 1 - )
#define USB_EPNTRST_IN(EPNTRSTN) (((USB->EPNTRSTN == 1)? 1 : 0)
//  ,   NAK  Host,  OUT_DATA(0 - ; 1 - )
#define USB_EPNTRST_OUT(EPNTRSTN) (((USB->EPNTRSTN == 2)? 1 : 0)
//     
#define USB_CR(EN_GLOB, LINEST, DIRCR, POL, SPEED, PULL) USB->CR = (EN_GLOB & 0x01)|((LINEST & 0x03) << 1)|((DIRCR & 0x01) << 3)|((POL & 0x01) << 4)|((SPEED & 0x01) << 5)|((PULL & 0x01) << 6)
//   (0 - ; 1 - )
#define USB_LNST_RESET() ((((USB->LNST) & 0x03) == 0)? 1 : 0)
//   1,5 /(0 - ; 1 - )
#define USB_LNST_1_5() ((((USB->LNST) & 0x03) == 1)? 1 : 0)
//   12 /(0 - ; 1 - )
#define USB_LNST_12() ((((USB->LNST) & 0x03) == 2)? 1 : 0)
//   USB +5V (0 - ; 1 - )
#define USB_LNST_VBUS() (((USB->LNST) & 0x04)? 1 : 0)
//  INT_NUM  
#define USB_INT_CLR(INT_NUM) ((USB->INT) |= (1 << INT_NUM))
//  
#define USB_RX_CLR(RXCLRN) ((USB->RXCLRN) |= 1 )
//  
#define USB_TX_CLR(TXCLRN) ((USB->TXCLRN) |= 1 )

#endif






